/*
 * DESCRIPTION:
 * camera.c header.
*/

#pragma once

#include <GL/glew.h>
#include <GL/gl.h>

#include "window.h"

#define CAMERA_ORTHOGRAPHIC_FAR -1.0f
#define CAMERA_ORTHOGRAPHIC_NEAR 1.0f
#define CAMERA_PERSPECTIVE_FAR 100.0f
#define CAMERA_PERSPECTIVE_NEAR 0.1f
#define CAMERA_PERSPECTIVE_YAW_DEFAULT (-M_PI / 2)

enum CameraType
{
	CAMERA_PERSPECTIVE,
	CAMERA_ORTHOGRAPHIC
};

struct Camera
{
	enum CameraType type;
	vec3 position;
	f32 aspect;
	f32 fov;
	vec2 size;
	vec3 front;
	vec3 up;
	vec3 right;
	f32 yaw;
	f32 pitch;
	mat4 view;
	mat4 projection;
};

void camera_perspective_init(struct Camera* self, f32 fov, f32 aspect);
void camera_orthographic_init(struct Camera* self, vec2 size);
void camera_orthographic_size_set(struct Camera* self, vec2 size);
void camera_orthographic_window_set(struct Camera* self, struct Window* window);
void camera_position_set(struct Camera* self, vec3 position);
void camera_projection_set(struct Camera* self);
void camera_view_set(struct Camera* self);
void camera_vector_set(struct Camera* self);
void camera_update(struct Camera* self);
