/*
 * DESCRIPTION:
 * Event handling.
*/

#include "event.h"

/* Updates events. */
void
event_update(struct Event* self)
{
        SDL_Event event;

	self->isWindowLeft = false;
	self->isWindowResized = false;

        memcpy(&self->prev, &self->curr, sizeof(bool) * EVENT_COUNT);
        memset(&self->curr, '\0', sizeof(bool) * EVENT_COUNT);

        while (SDL_PollEvent(&event))
        {
                switch (event.type)
                {
			case SDL_WINDOWEVENT:
				switch (event.window.event)
				{
					case SDL_WINDOWEVENT_LEAVE:
						self->isWindowLeft = true;
						break;
					case SDL_WINDOWEVENT_RESIZED:
        				case SDL_WINDOWEVENT_SIZE_CHANGED:
						self->isWindowResized = true;
					default:
						break;
				}
				break;
                        case SDL_QUIT:
                                self->curr[EVENT_QUIT] = true;
                                break;
                        default:
                                break;
                }
        }
}

/* Is a given event pressed? */
bool
event_press(struct Event* self, enum EventType type)
{
        return (self->curr[type] && !self->prev[type]);
}

/* Is a given event held? */
bool
event_held(struct Event* self, enum EventType type)
{
        return (self->curr[type] && self->prev[type]);
}

/* Is a given event released? */
bool
event_release(struct Event* self, enum EventType type)
{
        return (!self->curr[type] && self->prev[type]);
}
