/*
 * DESCRIPTION:
 * Mouse functions.
*/

#include "mouse.h"

/* Updates the mouse. */
void
mouse_update(struct Mouse* self)
{
        s32 state;
	s32 x;
	s32 y;

        memcpy(&self->prev, &self->curr, sizeof(bool) * MOUSE_COUNT);
        memset(&self->curr, '\0', sizeof(bool) * MOUSE_COUNT);

        state = SDL_GetMouseState(NULL, NULL);

        if ((state & SDL_BUTTON_LMASK) != 0)
                self->curr[MOUSE_LEFT] = true;

        if ((state & SDL_BUTTON_MMASK) != 0)
                self->curr[MOUSE_MIDDLE] = true;

        if ((state & SDL_BUTTON_RMASK) != 0)
                self->curr[MOUSE_RIGHT] = true;

	SDL_GetMouseState(&x, &y);

	self->position[0] = (f32)x;
	self->position[1] = (f32)y;
}

/* Is a given mouse button pressed? */
bool
mouse_press(struct Mouse* self, enum MouseType type)
{
        return (self->curr[type] && !self->prev[type]);
}

/* Is a given mouse button held? */
bool
mouse_held(struct Mouse* self, enum MouseType type)
{
        return (self->curr[type] && self->prev[type]);
}

/* Is a given mouse button released? */
bool
mouse_release(struct Mouse* self, enum MouseType type)
{
        return (!self->curr[type] && self->prev[type]);
}
