/*
 * DESCRIPTION:
 * Basic rendering.
*/

#include "renderer.h"

/* Initializes renderer. */
void
renderer_init(struct Renderer* self, struct Window* window, enum CameraType type)
{
	self->window = window;

	self->glContext = SDL_GL_CreateContext(self->window->sdl);

	glew_init();

	vao_init(&self->vao);
	vbo_init(&self->vbo, GL_ARRAY_BUFFER, true);
	vbo_init(&self->ebo, GL_ELEMENT_ARRAY_BUFFER, true);

	switch (type)
	{
		case CAMERA_PERSPECTIVE:
			camera_perspective_init
			(
				&self->camera, 
				RENDERER_CAMERA_PERSPECTIVE_FOV_DEFAULT, 
				RENDERER_CAMERA_PERSPECTIVE_ASPECT_DEFAULT
			);
			break;
		case CAMERA_ORTHOGRAPHIC:
			camera_orthographic_init(&self->camera, self->window->size);
			break;
		default:
			break;
	}
}

/* Frees renderer. */
void
renderer_free(struct Renderer* self)
{
	memset(self, '\0', sizeof(struct Renderer));
	SDL_GL_DeleteContext(self->glContext);
}

/* Sets renderer's viewport. */
void
renderer_viewport_set(struct Renderer* self, ivec4 viewport)
{
	glViewport(viewport[0], viewport[1], viewport[2], viewport[3]);
}

/* Sets the renderer's viewport to its window. */
void
renderer_viewport_window_set(struct Renderer* self)
{
	ivec4 viewport;

	window_ivec4_get(self->window, viewport);

	renderer_viewport_set(self, viewport);
}

/* Uses the specified shader. */
void
renderer_shader_use(struct Renderer* self, struct Shader* shader)
{
	self->shader = shader;
	shader_use(shader);
}

/* Sets the renderer clear color. */
void
renderer_clear_color_set(struct Renderer* self, vec4 color)
{
	glClearColor(color[0], color[1], color[2], color[3]);
	
	glm_vec4_copy(color, self->clearColor);
}

/* Clears the renderer. */
void
renderer_clear(struct Renderer* self)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

/* Presents renderer. */
void
renderer_present(struct Renderer* self)
{
	SDL_GL_SwapWindow(self->window->sdl);
}

/* Updates renderer. */
void
renderer_update(struct Renderer* self)
{
	renderer_viewport_window_set(self);
	camera_update(&self->camera);

	if (self->camera.type == CAMERA_ORTHOGRAPHIC)
		camera_orthographic_window_set(&self->camera, self->window);
}


