/*
 * DESCRIPTION:
 * OpenGL texture abstraction.
*/

#include "texture.h"

/* Given an SDL_Surface, creates a GL texture from it. */
/* NOTE: loading only supports RGBA */
bool
texture_surface_init(struct Texture* self, struct SDL_Surface* surface)
{
	if (!self->isInit)
	{
		glGenTextures(1, &self->handle);
		self->isInit = true;
	}

	if (!surface)
		return false;

	self->w = surface->w;
	self->h = surface->h;

	texture_bind(self);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, self->w, self->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, surface->pixels);

	glGenerateMipmap(GL_TEXTURE_2D);

	return true;
}

/* Initializes the OpenGL texture, given image data. */
bool
texture_init(struct Texture* self, char* path)
{
	struct SDL_Surface* surface;

	surface = IMG_Load(path);

	if (!surface)
		return false;

	texture_surface_init(self, surface);

	SDL_FreeSurface(surface);

	return true;
}

/* Binds a texture. */
void
texture_bind(struct Texture* self)
{
	glBindTexture(GL_TEXTURE_2D, self->handle);
}

/* Frees texture. */
void
texture_free(struct Texture* self)
{
	glDeleteTextures(1, &self->handle);
	memset(self, '\0', sizeof(struct Texture));
}
