/*
 * DESCRIPTION:
 * Window functionality.
*/

#include "window.h"

/* Initializes a window; needs menu, width, height, and flags. */
void
window_init(struct Window* self, char* name, vec2 size, u32 flags)
{
	self->name = name;
	self->flags = flags;

	glm_vec2_copy(size, self->size);

	self->isFullscreen = false;

	self->sdl = SDL_CreateWindow
	(
		name,
		SDL_WINDOWPOS_UNDEFINED,
		SDL_WINDOWPOS_UNDEFINED,
		(s32)self->size[0],
		(s32)self->size[1],
		flags
	);
}

/* Frees a given window. */
void
window_free(struct Window* self)
{
	SDL_DestroyWindow(self->sdl);
	
	memset(self, '\0', sizeof(struct Window));
}

/* Returns window size. */
void
window_ivec4_get(struct Window* self, ivec4 vector)
{
	vector[0] = 0;
	vector[1] = 0;
	vector[2] = self->size[0];
	vector[3] = self->size[1];
}

/* Sets video mode to fullscreen. */
void
window_fullscreen_enter(struct Window* self)
{
	SDL_DisplayMode dm;
	s32 w;
	s32 h;

	SDL_GetCurrentDisplayMode(0, &dm);

	w = dm.w;
	h = dm.h;

	SDL_SetWindowSize(self->sdl, w, h);

	self->isFullscreen = true;

	SDL_SetWindowFullscreen(self->sdl, SDL_WINDOW_FULLSCREEN);
}

/* Exit fullscreen.*/
void
window_fullscreen_exit(struct Window* self)
{
	self->isFullscreen = false;

	SDL_SetWindowFullscreen(self->sdl, 0);
}

void
window_fullscreen_toggle(struct Window* self)
{
	if (!self->isFullscreen)
		window_fullscreen_enter(self);
	else
		window_fullscreen_exit(self);
}

/* Updates window. */
void
window_update(struct Window* self)
{
	s32 w;
	s32 h;

	SDL_GetWindowSize(self->sdl, &w, &h);

	self->size[0] = (f32)w;
	self->size[1] = (f32)h;
}

/* Window coords -> OpenGL coords. */
void
window_coords_to_gl(struct Window* self, vec2 position, vec2 dest)
{
	dest[0] = ((position[0] / (f32)self->size[0]) * 2.0f) - 1.0f;
	dest[1] = ((position[1] / (f32)self->size[1]) * 2.0f) - 1.0f;
}
