/*
 * DESCRIPTION:
 * Chase component.
*/

#include "c_chase.h"

static void update_f(struct CChase* self);

/* Draws a chase component. */
static void
update_f(struct CChase* self)
{
	struct CPhysics* physics;
	struct CPhysics* targetPhysics;
	f32 angle;

	physics = ecs_get(self->id, ECS_C_PHYSICS);
	targetPhysics = ecs_get(self->targetID, ECS_C_PHYSICS);

	if (!physics || !targetPhysics)
		return;

	angle = ATAN
	(
		targetPhysics->position[0],
		physics->position[0],
		targetPhysics->position[1],
		physics->position[1]
	);

	physics->velocity[0] -= (cos(angle) * self->speed);
	physics->velocity[1] -= (sin(angle) * self->speed);
}

/* Sets chase component values. */
void
c_chase_set(struct CChase* self, u32 targetID, f32 speed)
{
	self->targetID = targetID;
	self->speed = speed;
}

/* Registers chase component. */
void
c_chase_register(void)
{
	ecs_register
	(
		ECS_C_CHASE,
		sizeof(struct CChase),
		C_CHASE_LIMIT,
		NULL,
		NULL,
		(ECSFunction)update_f,
		NULL,
		NULL
	);
}
