/*
 * DESCRIPTION:
 * Circle collide component.
*/

#include "c_circle_collide.h"

static bool collide(struct CCircle* a, struct CCircle* b);
static void add(struct CCircleCollide* self);
static void update_f(struct CCircleCollide* self);
static void delete(struct CCircleCollide* self);

/* Are two circle components colliding? */
static bool
collide(struct CCircle* a, struct CCircle* b)
{
	struct CPhysics* aPhysics;
	struct CPhysics* bPhysics;
	f32 distance;
	f32 sum;

	aPhysics = ecs_get(a->id, ECS_C_PHYSICS);
	bPhysics = ecs_get(b->id, ECS_C_PHYSICS);

	if (!aPhysics || !bPhysics)
		return false;

	distance = DISTANCE_2D
	(
		aPhysics->position[0],
		bPhysics->position[0],
		aPhysics->position[1],
		bPhysics->position[1]
	);

	sum = a->radius + b->radius;

	if (distance < sum)
		return true;

	return false;
}

/* Adds circle collide component. */
static void
add(struct CCircleCollide* self)
{
	vector_init(&self->collisions, sizeof(u32), C_CIRCLE_COLLIDE_COLLISIONS_LIMIT);
}

/* Deletes circle collide component. */
static void
delete(struct CCircleCollide* self)
{
	vector_free(&self->collisions);
}

/* Draws a circle collide component. */
static void
update_f(struct CCircleCollide* self)
{
	struct CCircle* circle;
	
	circle = ecs_get(self->id, ECS_C_CIRCLE);

	vector_clear(&self->collisions);

	if (!circle)
		return;

	for (s32 i = 0; i < (s32)game.ecs.lists[ECS_C_CIRCLE].components.count; i++)
	{
		struct CCircle* collideCircle;

		collideCircle = ecs_from_index_get(i, ECS_C_CIRCLE);

		if (collideCircle->id == self->id)
			continue;

		if (collide(circle, collideCircle))
			vector_push(&self->collisions, &collideCircle->id);
	}
}

/* Registers circle collide component. */
void
c_circle_collide_register(void)
{
	ecs_register
	(
		ECS_C_CIRCLE_COLLIDE,
		sizeof(struct CCircleCollide),
		C_CIRCLE_COLLIDE_LIMIT,
		(ECSFunction)add,
		(ECSFunction)delete,
		(ECSFunction)update_f,
		NULL,
		NULL
	);
}
