/*
 * DESCRIPTION:
 * Color change component.
*/

#include "c_color_change.h"

static void add(struct CColorChange* self);
static void update_f(struct CColorChange* self);

/* Adds a color change component. */
static void
add(struct CColorChange* self)
{
	self->isFinished = true;
}

/* Updates an color change component. */
static void
update_f(struct CColorChange* self)
{
	f32 progress;
	struct CSprite* sprite;
	struct CText* text;
	vec4 current;
	vec4 difference;
	vec4 step;
	vec4* color;

	sprite = ecs_get(self->id, ECS_C_SPRITE);
	text = ecs_get(self->id, ECS_C_TEXT);

	if (self->isFinished)
		return;

	if (sprite)
		color = &sprite->color;
	else if (text)
		color = &text->color;
	else
		return;

	progress = (f32)self->timer / self->timerMax;

	switch (self->type)
	{
		case C_COLOR_CHANGE_TO:
		case C_COLOR_CHANGE_TO_STICK:
			glm_vec4_copy(self->baseColor, current);
			glm_vec4_sub(current, self->color, difference);
			glm_vec4_scale(difference, (1.0f - progress), step);
			glm_vec4_sub(current, step, current);
			glm_vec4_copy(current, *color);
			break;
		case C_COLOR_CHANGE_FROM:
		case C_COLOR_CHANGE_FROM_STICK:
			glm_vec4_copy(self->color, current);
			glm_vec4_sub(current, self->baseColor, difference);
			glm_vec4_scale(difference, (1.0f - progress), step);
			glm_vec4_sub(current, step, current);
			glm_vec4_copy(current, *color);
			break;
		default:
			break;
	}

	self->timer--;

	if (self->timer <= 0)
	{
		switch (self->type)
		{
			case C_COLOR_CHANGE_TO:
			case C_COLOR_CHANGE_FROM:
				glm_vec4_copy(self->baseColor, *color);
				break;
			case C_COLOR_CHANGE_TO_STICK:
			case C_COLOR_CHANGE_FROM_STICK:
			default:
				break;
		}

		self->isFinished = true;
	}
}

/* Sets color_change component. */
void
c_color_change_set(struct CColorChange* self, vec4 baseColor, vec4 color, s32 timerMax, enum CColorChangeType type)
{
	glm_vec4_copy(baseColor, self->baseColor);
	glm_vec4_copy(color, self->color);

	self->timerMax = timerMax;
	self->type = type;

	self->timer = self->timerMax;
	self->isFinished = false;
}

/* Registers color_change component. */
void
c_color_change_register(void)
{
	ecs_register
	(
		ECS_C_COLOR_CHANGE,
		sizeof(struct CColorChange),
		C_COLOR_CHANGE_LIMIT,
		(ECSFunction)add,
		NULL,
		(ECSFunction)update_f,
		NULL,
		NULL
	);
}
