/*
 * DESCRIPTION:
 * Physics component.
*/

#include "c_control.h"

static void update_f(struct CControl* self);

/* Draws a control component. */
static void
update_f(struct CControl* self)
{
	struct CPhysics* physics;
	vec2 glMouse;
	vec2 glPosition;
	f32 distance;
	f32 angle;
	f32 travel;

	physics = ecs_get(self->id, ECS_C_PHYSICS);

	if (!physics || self->isDisabled)
		return;

	window_coords_to_gl(&game.window, game.mouse.position, glMouse); 
	window_coords_to_gl(&game.window, physics->position, glPosition); 

	distance = DISTANCE_2D(glPosition[0], glMouse[0], glPosition[1], glMouse[1]);

	if (distance < self->moveMinimum)
		return;

	travel = distance / self->speed;

	angle = ATAN(glPosition[0], glMouse[0], glPosition[1], glMouse[1]);

	physics->velocity[0] += cos(angle) * travel;
	physics->velocity[1] += sin(angle) * travel;
}

/* Sets control component values. */
void
c_control_set(struct CControl* self, f32 speed, f32 moveMinimum)
{
	self->speed = speed;
	self->moveMinimum = moveMinimum;
}

/* Registers control component. */
void
c_control_register(void)
{
	ecs_register
	(
		ECS_C_CONTROL,
		sizeof(struct CControl),
		C_CONTROL_LIMIT,
		NULL,
		NULL,
		(ECSFunction)update_f,
		NULL,
		NULL
	);
}
