/*
 * DESCRIPTION:
 * Damage component.
*/

#include "c_damage.h"

static void update_f(struct CDamage* self);

/* Updates a damage component. */
static void
update_f(struct CDamage* self)
{
	struct CHealth* health;
	struct CFlash* flash;
	struct CStun* stun;

	health = ecs_get(self->id, ECS_C_HEALTH);

	self->isDamageUpdate = false;

	if (!health)
		return;

	if (self->timer > 0)
	{
		self->timer--;
		self->isDamaged = false;
		return;
	}

	if (!self->isDamaged || health->isDead)
		return;

	self->timer = C_DAMAGE_TIMER;

	flash = ecs_get(self->id, ECS_C_FLASH);
	stun = ecs_get(self->id, ECS_C_STUN);

	if (!flash || !stun)
		return;

	self->isDamageUpdate = true;

	c_flash_set(flash, game.renderer.clearColor, C_DAMAGE_FLASH_COLOR, C_DAMAGE_TIMER, C_FLASH_STROBE);

	c_stun_set(stun, C_DAMAGE_TIMER);

	health->current--;

	self->isDamaged = false;

	sound_play(&game.sounds[SOUND_HURT], -1);
}

/* Registers damage component. */
void
c_damage_register(void)
{
	ecs_register
	(
		ECS_C_DAMAGE,
		sizeof(struct CDamage),
		C_DAMAGE_LIMIT,
		NULL,
		NULL,
		(ECSFunction)update_f,
		NULL,
		NULL
	);
}
