/*
 * DESCRIPTION:
 * Game object component.
*/

#include "c_game_object.h"

static void update_f(struct CGameObject* self);

/* Draws a game object component. */
static void
update_f(struct CGameObject* self)
{
	struct CPhysics* physics;
	struct CSprite* sprite;

	physics = ecs_get(self->id, ECS_C_PHYSICS);
	sprite = ecs_get(self->id, ECS_C_SPRITE);

	if (!physics || !sprite)
		return;

	glm_vec3_copy(physics->position, sprite->position);
}

/* Sets game object component to specified values. */
void
c_game_object_set
(
	struct CGameObject* self, 
	struct Texture texture,
	enum ShaderType shader,
	enum OriginType origin,
	vec2 size,
	vec3 position,
	vec4 color,
	f32 friction,
	f32 velocityMax,
	f32 radius
)
{

	struct CPhysics* physics;
	struct CSprite* sprite;
	struct CCircle* circle;

	physics = ecs_add(self->id, ECS_C_PHYSICS);
	sprite = ecs_add(self->id, ECS_C_SPRITE);
	circle = ecs_add(self->id, ECS_C_CIRCLE);
	ecs_add(self->id, ECS_C_CIRCLE_COLLIDE);

	c_sprite_set(sprite, texture, shader, origin, size, color, position);
	c_physics_set(physics, position, friction, velocityMax);
	c_circle_set(circle, radius);
}

/* Sets game object component to specified values, plus additional texture atlas info. */
void
c_game_object_atlas_set
(
	struct CGameObject* self, 
	struct Texture texture,
	enum ShaderType shader,
	enum OriginType origin,
	s32 frameW,
	s32 frameH,
	s32 rows,
	s32 cols,
	vec2 size,
	vec3 position,
	vec4 color,
	f32 friction,
	f32 velocityMax,
	f32 radius
)
{
	struct CPhysics* physics;
	struct CSprite* sprite;
	struct CCircle* circle;

	physics = ecs_add(self->id, ECS_C_PHYSICS);
	sprite = ecs_add(self->id, ECS_C_SPRITE);
	circle = ecs_add(self->id, ECS_C_CIRCLE);
	ecs_add(self->id, ECS_C_CIRCLE_COLLIDE);

	c_sprite_atlas_set(sprite, texture, shader, origin, frameW, frameH, rows, cols, size, color, position);
	c_physics_set(physics, position, friction, velocityMax);
	c_circle_set(circle, radius);
}


/* Registers game_object component. */
void
c_game_object_register(void)
{
	ecs_register
	(
		ECS_C_GAME_OBJECT,
		sizeof(struct CGameObject),
		C_GAME_OBJECT_LIMIT,
		NULL,
		NULL,
		(ECSFunction)update_f,
		NULL,
		NULL
	);
}
