/*
 * DESCRIPTION:
 * Move component.
*/

#include "c_move.h"

static void add(struct CMove* self);
static void update_f(struct CMove* self);

static void
add(struct CMove* self)
{
	self->isFinished = true;
}

/* Updates an move component. */
static void
update_f(struct CMove* self)
{
	struct CSprite* sprite;
	struct CText* text;
	f32 progress;
	f32 distance;
	vec3* position;
	vec3 difference;
	vec3 step;
	vec3 current;

	sprite = ecs_get(self->id, ECS_C_SPRITE);
	text = ecs_get(self->id, ECS_C_TEXT);

	if (self->isFinished)
		return;

	if (sprite)
		position = &sprite->position;
	else if (text)
		position = &text->position;
	else
		return;

	progress = (f32)self->timer / self->timerMax;

	switch (self->type)
	{
		case C_MOVE_TO:
		case C_MOVE_TO_STICK:
			glm_vec3_copy(self->start, current);
			glm_vec3_sub(current, self->end, difference);
			glm_vec3_scale(difference, (1.0f - progress), step);
			glm_vec3_sub(current, step, current);
			glm_vec3_copy(current, *position);
			break;
		case C_MOVE_FROM:
		case C_MOVE_FROM_STICK:
			glm_vec3_copy(self->end, current);
			glm_vec3_sub(current, self->start, difference);
			glm_vec3_scale(difference, (1.0f - progress), step);
			glm_vec3_sub(current, step, current);
			glm_vec3_copy(current, *position);
				break;
		default:
			break;
	}

	self->timer--;

	if (self->timer <= 0)
	{
		switch (self->type)
		{
			case C_MOVE_TO:
			case C_MOVE_FROM:
				glm_vec3_copy(self->start, *position);
				break;
			case C_MOVE_TO_STICK:
			case C_MOVE_FROM_STICK:
			default:
				break;
		}

		self->isFinished = true;
	}
}

/* Sets move component. */
void
c_move_set(struct CMove* self, vec3 start, vec3 end, s32 timerMax, enum CMoveType type)
{
	glm_vec3_copy(start, self->start);
	glm_vec3_copy(end, self->end);

	self->timerMax = timerMax;
	self->type = type;

	self->timer = self->timerMax;
	self->isFinished = false;
}

/* Registers move component. */
void
c_move_register(void)
{
	ecs_register
	(
		ECS_C_MOVE,
		sizeof(struct CMove),
		C_MOVE_LIMIT,
		(ECSFunction)add,
		NULL,
		(ECSFunction)update_f,
		NULL,
		NULL
	);
}
