/*
 * DESCRIPTION:
 * ParticleSpawn component.
*/

#include "c_particle_spawn.h"

static f32 spin_speed_get(void);

static f32
spin_speed_get(void)
{
	return (RANDOM * (PARTICLE_SPAWN_SPIN_SPEED_MAX - PARTICLE_SPAWN_SPIN_SPEED_MIN) + PARTICLE_SPAWN_SPIN_SPEED_MIN);
}

/* Spawns a particle at a given location. */
void
c_particle_spawn(struct CParticleSpawn* self, vec3 position)
{
	u32 lilParticleID;
	u32 bigParticleID;
	f32 lilParticleSpinSpeed;
	f32 bigParticleSpinSpeed;

	lilParticleID = entity_add();
	bigParticleID = entity_add();

	lilParticleSpinSpeed = spin_speed_get();
	bigParticleSpinSpeed = spin_speed_get();

	particle_init
	(
		lilParticleID, 
		position,
		PARTICLE_SPAWN_LIL_PARTICLE_BASE_COLOR,
		COLOR_TRANSPARENT,
		PARTICLE_SPAWN_LIL_PARTICLE_SIZE,
		PARTICLE_SPAWN_SCALE,
		lilParticleSpinSpeed,
		PARTICLE_SPAWN_TIMER
	);

	particle_init
	(
		bigParticleID, 
		position,
		PARTICLE_SPAWN_BIG_PARTICLE_BASE_COLOR,
		COLOR_TRANSPARENT,
		PARTICLE_SPAWN_BIG_PARTICLE_SIZE,
		PARTICLE_SPAWN_SCALE,
		bigParticleSpinSpeed,
		PARTICLE_SPAWN_TIMER
	);
}

/* Registers particle_spawn component. */
void
c_particle_spawn_register(void)
{
	ecs_register
	(
		ECS_C_PARTICLE_SPAWN,
		sizeof(struct CParticleSpawn),
		C_PARTICLE_SPAWN_LIMIT,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL
	);
}
