/*
 * DESCRIPTION:
 * Physics component.
*/

#include "c_physics.h"

static void update_f(struct CPhysics* self);

/* Draws a physics component. */
static void
update_f(struct CPhysics* self)
{
	glm_vec3_add(self->position, self->velocity, self->position);
	glm_vec3_scale(self->velocity, self->friction, self->velocity);

	self->velocity[0] = CLAMP(self->velocity[0], -self->velocityMax, self->velocityMax);
	self->velocity[1] = CLAMP(self->velocity[1], -self->velocityMax, self->velocityMax);
	self->velocity[2] = CLAMP(self->velocity[2], -self->velocityMax, self->velocityMax);
}

/* Sets physics component. */
void
c_physics_set(struct CPhysics* self, vec3 position, f32 friction, f32 velocityMax)
{
	glm_vec3_copy(position, self->position);

	self->friction = friction;
	self->velocityMax = velocityMax;
}

/* Registers physics component. */
void
c_physics_register(void)
{
	ecs_register
	(
		ECS_C_PHYSICS,
		sizeof(struct CPhysics),
		C_PHYSICS_LIMIT,
		NULL,
		NULL,
		(ECSFunction)update_f,
		NULL,
		NULL
	);
}
