/*
 * DESCRIPTION:
 * Pulsate component.
*/

#include "c_pulsate.h"

static void update_f(struct CPulsate* self);

/* Updates an pulsate component. */
static void
update_f(struct CPulsate* self)
{
	struct CSprite* sprite;
	f32 progress;
	f32 scale;

	sprite = ecs_get(self->id, ECS_C_SPRITE);

	if (!sprite || self->isDisabled)
		return;

	progress = (f32)self->timer / self->timerMax;

	sprite->scale = (sin(TAU * progress) * self->value) + self->baseScale;

	self->timer--;

	if (self->timer <= 0)
	{
		sprite->scale = self->baseScale;
		self->timer = self->timerMax;
	}
}

/* Sets pulsate component. */
void
c_pulsate_set(struct CPulsate* self, f32 baseScale, f32 value, s32 timerMax)
{
	self->baseScale = baseScale;
	self->timerMax = timerMax;
	self->value = value;

	self->timer = self->timerMax;
}

/* Registers pulsate component. */
void
c_pulsate_register(void)
{
	ecs_register
	(
		ECS_C_PULSATE,
		sizeof(struct CPulsate),
		C_PULSATE_LIMIT,
		NULL,
		NULL,
		(ECSFunction)update_f,
		NULL,
		NULL
	);
}
