/*
 * DESCRIPTION:
 * UI button component.
*/

#include "c_ui_button.h"

static void update_f(struct CUIButton* self);

/* Draws a ui_button component. */
static void
update_f(struct CUIButton* self)
{
	struct CSprite* sprite;
	struct CText* text;
	struct CColorChange* colorChange;
	vec4 rectangle;

	sprite = ecs_get(self->id, ECS_C_SPRITE);
	text = ecs_get(self->id, ECS_C_TEXT);
	colorChange = ecs_get(self->id, ECS_C_COLOR_CHANGE);

	if (self->isDisabled)
		return;

	if (sprite)
		c_sprite_rectangle_get(sprite, rectangle);
	else if (text)
		c_text_rectangle_get(text, rectangle);
	else
		return;

	if (rectangle_has_point(rectangle, game.mouse.position))
	{
		if (self->state != C_UI_BUTTON_HOVER)
		{
			c_color_change_set
			(
				colorChange,
				self->selectColor,
				self->baseColor,
				1,
				C_COLOR_CHANGE_TO_STICK
			);

			if (self->state != C_UI_BUTTON_HELD)
				sound_play(&game.sounds[SOUND_HOVER], -1);

		}

		self->state = C_UI_BUTTON_HOVER;

		if (mouse_held(&game.mouse, MOUSE_LEFT))
		{
			if (!self->isPressed)
			{
				sound_play(&game.sounds[SOUND_SELECT], -1);
				self->isPressed = true;
			}

			self->state = C_UI_BUTTON_HELD;
		}
	}
	else
	{
		if (self->state != C_UI_BUTTON_IDLE)
		{
			c_color_change_set
			(
				colorChange,
				self->baseColor,
				self->selectColor,
				1,
				C_COLOR_CHANGE_TO_STICK
			);

			self->isPressed = false;
		}
		self->state = C_UI_BUTTON_IDLE;
	}
}

/* Sets ui button component. */
void
c_ui_button_set(struct CUIButton* self, vec4 baseColor, vec4 selectColor)
{
	glm_vec4_copy(baseColor, self->baseColor);
	glm_vec4_copy(selectColor, self->selectColor);
}

/* Registers ui_button component. */
void
c_ui_button_register(void)
{
	ecs_register
	(
		ECS_C_UI_BUTTON,
		sizeof(struct CUIButton),
		C_UI_BUTTON_LIMIT,
		NULL,
		NULL,
		(ECSFunction)update_f,
		NULL,
		NULL
	);
}
