/*
 * DESCRIPTION:
 * Player code.
*/

#include "e_player.h"

/* Adds a follower to orbit the player. */
void
player_follower_add(u32 id)
{
	struct COrbit* orbit;
	u32 followerID;

	orbit = ecs_get(id, ECS_C_ORBIT);

	followerID = entity_add();

	follower_init(followerID, (enum CColorType)orbit->objects.count);

	c_orbit_object_add(orbit, followerID);
}

/* Initialize a player. */
void
player_init(u32 id, vec3 position)
{
	struct CGameObject* object;
	struct CControl* control;
	struct CRotation* rotation;
	struct CHealth* health;
	struct COrbit* orbit;
	struct CPulsate* pulsate;

	control = ecs_add(id, ECS_C_CONTROL);
	health = ecs_add(id, ECS_C_HEALTH);
	object = ecs_add(id, ECS_C_GAME_OBJECT);
	orbit = ecs_add(id, ECS_C_ORBIT);
	pulsate = ecs_add(id, ECS_C_PULSATE);
	rotation = ecs_add(id, ECS_C_ROTATION);
	ecs_add(id, ECS_C_COLOR_CHANGE);
	ecs_add(id, ECS_C_DAMAGE);
	ecs_add(id, ECS_C_FLASH);
	ecs_add(id, ECS_C_PLAYER_ATLAS_HEALTH);
	ecs_add(id, ECS_C_PLAYER_DEATH);
	ecs_add(id, ECS_C_SCALE);
	ecs_add(id, ECS_C_STUN);

	c_game_object_atlas_set
	(
		object, 
		game.textures[TEXTURE_PLAYER],
		PLAYER_SHADER,
		PLAYER_ORIGIN,
		PLAYER_ATLAS_FRAME_WIDTH,
		PLAYER_ATLAS_FRAME_HEIGHT,
		PLAYER_ATLAS_ROWS,
		PLAYER_ATLAS_COLUMNS,
		PLAYER_SIZE,
		position,
		PLAYER_COLOR,
		PLAYER_FRICTION,
		PLAYER_VELOCITY_MAX,
		PLAYER_RADIUS
	);
	
	c_control_set(control, PLAYER_CONTROL_SPEED, PLAYER_CONTROL_MOVE_MINIMUM);

	c_rotation_set(rotation, PLAYER_ROTATION_SPEED, PLAYER_ROTATION_SPEED_MAX, PLAYER_ROTATION_FRICTION);

	c_health_set(health, PLAYER_HEALTH_MAX);

	c_orbit_set(orbit, PLAYER_ORBIT_OFFSET);

	c_pulsate_set(pulsate, PLAYER_PULSATE_BASE_SCALE, PLAYER_PULSATE_VALUE, PLAYER_PULSATE_TIMER_MAX);
}
