/*
 * DESCRIPTION:
 * Main game functions.
*/

#include "game.h"

static void game_update_f(void);
static void game_update_v(void);
static void game_draw(void);

/* Updates the game on a f timestep. */
static void 
game_update_f(void)
{
	if (event_press(&game.event, EVENT_QUIT))
		game.isQuit = true;
	
	if (keyboard_press(&game.keyboard, KEYBOARD_M))
	{
		if (game.isMute)
			music_pause();
		else
			music_resume();

		game.isMute = !game.isMute;
	}

	event_update(&game.event);
	mouse_update(&game.mouse);
	keyboard_update(&game.keyboard);

	game_state_update_f();

	if (game.isPaused)
		return;

	ecs_update_f();
}

/* Updates game on a variable timestep. */
static void
game_update_v(void)
{
	window_update(&game.window);

	renderer_update(&game.renderer);

	ecs_update_v();
}

/* Draws the game. */
static void
game_draw(void)
{
	renderer_clear(&game.renderer);
	
	ecs_draw();
	
	renderer_present(&game.renderer);
}
/* Initializes game. */
void
game_init(void)
{
	memset(&game, '\0', sizeof(struct Game));

	RANDOM_SEED_SET(time(NULL));

	glm_vec4_copy(GAME_BOUNDS, game.bounds);

	game_shader_data_init();

	sdl_init();

	window_init(&game.window, WINDOW_MENU, WINDOW_SIZE, WINDOW_FLAGS);

	renderer_init(&game.renderer, &game.window, CAMERA_ORTHOGRAPHIC);
	renderer_clear_color_set(&game.renderer, RENDERER_CLEAR_COLOR);

	ecs_init();

	game.state = STATE_MENU;

	game_state_data_init();
	game_state_init();
}

/* Frees game resources. */
void
game_free(void)
{
	game_state_free();
	game_state_data_free();
	
	ecs_free();
	
	renderer_free(&game.renderer);
	window_free(&game.window);

	sdl_quit();
}

/* Main game loop. */
void
game_loop(void)
{
	tick_update(&game.tick);

	while (game.tick.cum > FRAME_DELAY)
	{
		game_update_f();
		game.tick.cum -= FRAME_DELAY;
	}

	game_update_v();
	game_draw();
}
