/*
 * DESCRIPTION:
 * Handles loading/freeing game shaders.
*/

#include "game_shader.h"

static struct ShaderData shaderData[SHADER_COUNT];

/* Gets all shader paths and copies their data. */
void
game_shader_data_init(void)
{
	for (s32 i = 0; i < SHADER_COUNT; i++)
	{
		if (
	
			file_read
			(
				(char*)SHADER_PATHS[i].vertexPath, 
				(char*)shaderData[i].vertex,
				SHADER_BUFFER_SIZE
			)
		)
			printf("LOADED: %s\n", SHADER_PATHS[i].vertexPath);
		else
			printf("FAILED: %s\n", SHADER_PATHS[i].vertexPath);

		if (
			file_read
			(
				(char*)SHADER_PATHS[i].fragmentPath, 
				(char*)shaderData[i].fragment,
				SHADER_BUFFER_SIZE
			)
		)
			printf("LOADED: %s\n", SHADER_PATHS[i].fragmentPath);
		else
			printf("FAILED: %s\n", SHADER_PATHS[i].fragmentPath);
	}
}

/* Initializes a game shader. */
void
game_shader_init(enum ShaderType shader)
{
	if (shader_init(&game.shaders[shader], (char*)shaderData[shader].vertex, (char*)shaderData[shader].fragment))
		printf("LOADED: Shader #%i\n", shader);
	else
		printf("FAILED: Shader #%i\n", shader);
}

/* Frees a game shader. */
void
game_shader_free(enum ShaderType shader)
{
	shader_free(&game.shaders[shader]);

	printf("FREED: Shader #%i\n", shader);
}
