/*
 * DESCRIPTION:
 * game_state.c header.
*/

#pragma once

#include "state/s_play.h"
#include "state/s_menu.h"
#include "state/s_results.h"

#include "game_font.h"
#include "game_music.h"
#include "game_shader.h"
#include "game_sound.h"
#include "game_texture.h"

struct StateData
{
	enum FontType* fonts;
	enum MusicType* music;
	enum ShaderType* shaders;
	enum SoundType* sounds;
	enum TextureType* textures;
	s32 fontCount;
	s32 musicCount;
	s32 shaderCount;
	s32 soundCount;
	s32 textureCount;
};

static const struct StateData STATE_DATA[STATE_COUNT] = 
{
	{
		.textures = STATE_MENU_TEXTURES, 
		.textureCount = STATE_MENU_TEXTURE_COUNT, 
		.sounds = STATE_MENU_SOUNDS, 
		.soundCount = STATE_MENU_SOUND_COUNT,
		.music = NULL,
		.musicCount = 0,
		.shaders = STATE_MENU_SHADERS,
		.shaderCount = STATE_MENU_SHADER_COUNT,
		.fonts = STATE_MENU_FONTS,
		.fontCount = STATE_MENU_FONT_COUNT
	},
	{
		.textures = STATE_PLAY_TEXTURES, 
		.textureCount = STATE_PLAY_TEXTURE_COUNT, 
		.sounds = STATE_PLAY_SOUNDS, 
		.soundCount = STATE_PLAY_SOUND_COUNT,
		.music = STATE_PLAY_MUSIC, 
		.musicCount = STATE_PLAY_MUSIC_COUNT,
		.shaders = STATE_PLAY_SHADERS,
		.shaderCount = STATE_PLAY_SHADER_COUNT,
		.fonts = STATE_PLAY_FONTS,
		.fontCount = STATE_PLAY_FONT_COUNT
	},
	{
		.textures = NULL, 
		.textureCount = 0, 
		.sounds = STATE_RESULTS_SOUNDS, 
		.soundCount = STATE_RESULTS_SOUND_COUNT,
		.music = NULL,
		.musicCount = 0,
		.shaders = STATE_RESULTS_SHADERS,
		.shaderCount = STATE_RESULTS_SHADER_COUNT,
		.fonts = STATE_RESULTS_FONTS,
		.fontCount = STATE_RESULTS_FONT_COUNT
	}
};

void game_state_data_init(void);
void game_state_data_free(void);
void game_state_init(void);
void game_state_free(void);
void game_state_update_f(void);
