/*
 * DESCRIPTION:
 * Rendering with SHADER_TEXTER_QUAD.
*/

#include "shader_texture_quad.h"

/* Draw function for. */
void
shader_texture_quad_draw(struct Atlas* atlas, enum ShaderType type, mat4 model, vec2 size, vec4 color)
{
	struct Shader* shader;
	struct Renderer* renderer;
	vec2 uvMin;
	vec2 uvMax;

	shader = &game.shaders[type];
	renderer = &game.renderer;

	atlas_uv_get(atlas, uvMin, uvMax);

	f32 verticies[4][5] = 
	{
		{0.0f   , 0.0f   , 0.0f, uvMin[0], uvMin[1]},
		{0.0f   , size[1], 0.0f, uvMin[0], uvMax[1]},
		{size[0], size[1], 0.0f, uvMax[0], uvMax[1]},
		{size[0], 0.0f   , 0.0f, uvMax[0], uvMin[1]},
	};

	u32 indices[2][3] = 
	{
		{1, 2, 3},
		{0, 1, 3}
	};

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	renderer_shader_use(&game.renderer, shader);

	shader_uniform_mat4_set(shader, SHADER_TEXTURE_QUAD_UNIFORM_VIEW, renderer->camera.view);
	shader_uniform_mat4_set(shader, SHADER_TEXTURE_QUAD_UNIFORM_PROJECTION, renderer->camera.projection);
	shader_uniform_mat4_set(shader, SHADER_TEXTURE_QUAD_UNIFORM_MODEL, model);
	shader_uniform_vec4_set(shader, SHADER_TEXTURE_QUAD_UNIFORM_COLOR, color);
	shader_uniform_texture_set(shader, SHADER_TEXTURE_QUAD_UNIFORM_TEXTURE, &atlas->texture, 0);

	vao_bind(&game.renderer.vao);

	vbo_bind(&game.renderer.vbo);
	vbo_buffer(&game.renderer.vbo, sizeof(verticies), verticies);

	vbo_bind(&game.renderer.ebo);
	vbo_buffer(&game.renderer.ebo, sizeof(indices), indices);

	vertex_attribute_set(0, 3, GL_FLOAT, 5 * sizeof(f32), 0);
	vertex_attribute_set(1, 2, GL_FLOAT, 5 * sizeof(f32), 3 * sizeof(f32));

	vao_bind(&game.renderer.vao);

	glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, 0);
	
	vao_unbind();
}	
