/*
 * DESCRIPTION:
 * Menu game state.
*/

#include "s_menu.h"

static void menu_text_color_change(struct Menu* self, struct CText* text);
static void menu_button_color_update(struct Menu* self);
static void menu_button_update(struct Menu* self);

/* Changes the color of each text entity on the menu. */
static void 
menu_text_color_change(struct Menu* self, struct CText* text)
{
	struct CColorChange* colorChange;

	colorChange = ecs_get(text->id, ECS_C_COLOR_CHANGE);

	c_color_change_set
	(
		colorChange,
		MENU_TEXT_COLOR_CHANGE_BASE_COLOR,
		MENU_TEXT_COLOR,
		MENU_TEXT_COLOR_CHANGE_TIMER,
		MENU_TEXT_COLOR_CHANGE_TYPE
	);
}

/* Updates menu buttons. */
static void
menu_button_color_update(struct Menu* self)
{
	if (self->isMenuDone)
		return;

	struct CColorChange* colorChange;

	colorChange = ecs_get(self->logoID, ECS_C_COLOR_CHANGE);

	if (colorChange->isFinished)
		self->isLogoDone = true;

	if (self->isLogoDone)
	{
		struct CText* startText;
		struct CText* creditsText;
		struct CUIButton* startUIButton;
		startText = ecs_get(self->startID, ECS_C_TEXT);
		creditsText = ecs_get(self->creditsID, ECS_C_TEXT);
		startUIButton = ecs_get(self->startID, ECS_C_UI_BUTTON);

		menu_text_color_change(self, startText);
		menu_text_color_change(self, creditsText);

		startUIButton->isDisabled = false;

		self->isMenuDone = true;
	}
}

/* Updates buttons. */
static void
menu_button_update(struct Menu* self)
{
	struct CUIButton* startUIButton;

	startUIButton = ecs_get(self->startID, ECS_C_UI_BUTTON);

	if (startUIButton->isPressed)
		self->isStart = true;

	if (keyboard_press(&game.keyboard, KEYBOARD_SPACE))
		self->isStart = true;
}

/* Initializes menu. */
void
menu_init(struct Menu* self)
{
	struct CUIButton* startUIButton;
	struct CText* startText;
	struct CText* creditsText;

	memset(self, '\0', sizeof(struct Menu));

	self->logoID = entity_add();
	self->startID = entity_add();
	self->creditsID = entity_add();

	logo_init(self->logoID);

	ui_button_init
	(
		self->startID, 
		MENU_START_ORIGIN,
		game.fonts[FONT_BIG],
		MENU_START_STRING,
		strlen(MENU_START_STRING),
		MENU_TEXT_COLOR,
		MENU_TEXT_SELECT_COLOR,
		MENU_START_POSITION
	);

	text_init
	(
		self->creditsID,
		MENU_CREDITS_ORIGIN,
		game.fonts[FONT_SMALL],
		MENU_CREDITS_STRING,
		strlen(MENU_CREDITS_STRING),
		MENU_CREDITS_WRAP,
		MENU_TEXT_COLOR,
		MENU_CREDITS_POSITION
	);

	startUIButton = ecs_get(self->startID, ECS_C_UI_BUTTON);
	startText = ecs_get(self->startID, ECS_C_TEXT);
	creditsText = ecs_get(self->creditsID, ECS_C_TEXT);

	startText->color[3] = 0.0f;
	creditsText->color[3] = 0.0f;
	
	startUIButton->isDisabled = true;
}

/* Updates menu. */
void
menu_update_f(struct Menu* self)
{
	menu_button_color_update(self);
	menu_button_update(self);
}

/* Frees menu. */
void
menu_free(struct Menu* self)
{
	entity_delete(self->startID);
	entity_delete(self->logoID);
}
