/*
 * DESCRIPTION:
 * play.c header.
*/

#pragma once

#include "../ecs/e_enemy.h"
#include "../ecs/e_player.h"
#include "../ecs/e_text.h"
#include "../ecs/e_warning.h"

#define PLAY_ENEMY_LIMIT 64
#define PLAY_ENEMY_SPEED 0.1f
#define PLAY_ENEMY_SPEED_LEVEL_MULTIPLIER 0.025f

#define PLAY_LEVEL_SPAWN_TIMER_MULTIPLIER 2
#define PLAY_LEVEL_TIMER_BASE 1200

#define PLAY_SPAWN_TIMER_BASE 60
#define PLAY_SPAWN_TIMER_DEFAULT 0
#define PLAY_SPAWN_TIMER_MIN 5
#define PLAY_SPAWN_TIMER_MULTIPLIER_MAX 0.95
#define PLAY_SPAWN_TIMER_MULTIPLIER_MIN 0.05
#define PLAY_SPAWN_TIMER_STUN_TIME 180

#define PLAY_START_TIMER_DEFAULT 420

#define PLAY_LEVEL_BUFFER_SIZE 16
#define PLAY_LEVEL_FONT FONT_BIG
#define PLAY_LEVEL_ORIGIN ORIGIN_TOP_RIGHT
#define PLAY_LEVEL_STRING "x0"
#define PLAY_LEVEL_WRAP -1
static const vec3 PLAY_LEVEL_POSITION = {1264.0f, 16.0f, 0.0f};
static const vec4 PLAY_LEVEL_COLOR = {1.0f, 1.0f, 1.0f, 1.0f};

#define PLAY_SCORE_BUFFER_SIZE 16
#define PLAY_SCORE_COLOR C_COLOR_VALUES[0]
#define PLAY_SCORE_COLOR_CHANGE_TYPE C_COLOR_CHANGE_TO_STICK
#define PLAY_SCORE_FONT FONT_BIG
#define PLAY_SCORE_ORIGIN ORIGIN_TOP_LEFT
#define PLAY_SCORE_SCALE_BASE 1.0f
#define PLAY_SCORE_SCALE_TYPE C_SCALE_FROM
#define PLAY_SCORE_SCALE_VALUE 1.5f
#define PLAY_SCORE_STRING "0"
#define PLAY_SCORE_COLOR_CHANGE_TIMER 16
#define PLAY_SCORE_SCALE_TIMER 16
#define PLAY_SCORE_WRAP -1
static const vec3 PLAY_SCORE_POSITION = {16.0f, 16.0f, 0.0f};

#define PLAY_MESSAGE_STRING "Don't let anything touch your white center.\nDestroy attackers by touching them with their color.\n\nMove with your mouse.\nSpin with WASD/arrow keys.\n\nSurvive.\n"
#define PLAY_MESSAGE_ORIGIN ORIGIN_CENTER
#define PLAY_MESSAGE_WRAP -1
#define PLAY_MESSAGE_COLOR C_COLOR_VALUES[0]
#define PLAY_MESSAGE_FONT FONT_SMALL
static const vec3 PLAY_MESSAGE_POSITION = {640.0f, 360.0f, 0.0f};

static const vec3 PLAY_PLAYER_POSITION = {640.0f, 360.0f, 0.0f};


#define STATE_PLAY_TEXTURE_COUNT 5
static const enum TextureType STATE_PLAY_TEXTURES[STATE_PLAY_TEXTURE_COUNT] = 
{
	TEXTURE_PLAYER,
	TEXTURE_FOLLOWER,
	TEXTURE_ENEMY,
	TEXTURE_PARTICLE,
	TEXTURE_WARNING
};

#define STATE_PLAY_SOUND_COUNT 4
static const enum SoundType STATE_PLAY_SOUNDS[STATE_PLAY_SOUND_COUNT] = 
{
	SOUND_COLLIDE,
	SOUND_LEVEL,
	SOUND_HURT,
	SOUND_WARNING
};

#define STATE_PLAY_MUSIC_COUNT 1
static const enum MusicType STATE_PLAY_MUSIC[STATE_PLAY_MUSIC_COUNT] = 
{
	MUSIC_ETHNICA
};

#define STATE_PLAY_SHADER_COUNT 1
static const enum ShaderType STATE_PLAY_SHADERS[STATE_PLAY_SHADER_COUNT] = 
{
	SHADER_TEXTURE_QUAD
};

#define STATE_PLAY_FONT_COUNT 2
static const enum FontType STATE_PLAY_FONTS[STATE_PLAY_FONT_COUNT] = 
{
	FONT_SMALL,
	FONT_BIG
};

void play_init(struct Play* self);
void play_update_f(struct Play* self);
void play_free(struct Play* self);
